#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include "blowfish.h"
#include "defs.h"

#ifndef __DB_H__
#define __DB_H__

struct dbUsers
{
        char CryptID[4];                      /* Used by crypt functions */
        int Host;                             /* User's host */
        char UserName[MAX_USERNAME_SIZE + 1]; /* User's name */
        char Password[MAX_PASSWORD_SIZE + 1]; /* User's password */
        char Comment[MAX_COMMENT_SIZE + 1];   /* User's comment */
};

struct dbHosts
{
        char CryptID[4];                      /* Used by crypt functions */
        int Host;                             /* Host's ID */
        char HostName[MAX_HOSTNAME_SIZE + 1]; /* Host's Name */
        char Comment[MAX_COMMENT_SIZE + 1];   /* Host's comment */
};

/*
   Generate Key function.
   Results:
   -1: Database not exist
   0: Wrong password
   1: Right password
*/
int generateKey(BF_KEY *Key,            /* The Key which will be generated*/
                char *Password,         /* The Password */
		int md5                 /* TRUE=Generate MD5 Hash, FALSE=Old style */
                );

/*
   Converts an error no to a string
   Valid codes are:
     0: No Error
    -1: Database does not exist yet
    -2: Decryption error (Wrong Password?)
    -3: Host already exist
    -4: Host does not exist
    -5: Hostname is not valid
    -6: Cannot delete because users exist
   -10: MAX_HOSTS reached
   -11: User already exist
   -12: User does not exist
   -13: Username is not valid
   -20: MAX_USERS for Host reached
*/
char *errorToStr(int error);

/*
   Get the HostDB filename depending on UserName given.
   Results
   0: UserName not exist
   1: all ok
*/
int getHostDB(char *HostDB,     /* "char Path[256]" should be enough */
              char *UserName    /* UserName. NULL for current user */
              );

/*
   Exist a host in the database and if not,
   exist an deleted host, that we can override?
   Results:
   > -1 : Host not exist, but an deleted host was found
   -1   : Database does not exist yet
   -2   : Wrong Key
   -3   : Host exist
   -4   : Host not exist, no deleted host exist
*/
int existHostEntry(struct dbHosts *Host,        /* Host structure. For the search, only HostName is used.
                                                   Host will be filled, if a deleted Host exist */
                   BF_KEY *Key                  /* Key generated by generateKey() */
                   );

/*
   Get the position of a host in the database
   Results:
   > -1 : Host was found on position <result>
   -1   : Database does not exist yet
   -2   : Wrong Key
   -4   : Host not exist
*/
int findHostEntry(struct dbHosts *Host,         /* Host structure. For the search, only HostName is used.
                                                   Host will be filled, if the host exist */
                  BF_KEY *Key                   /* Key generated by generateKey() */
                  );

/*
   Get a dbHosts structure from the HostID
   Error if the Host member of the structure is < 0
   -1   : Database does not exist
   -2   : Wrong key
   -4   : Host not exist
*/
struct dbHosts getHostName(int HostID,          /* The HostID that is searched */
                           BF_KEY *Key          /* Key generated by generateKey() */
                           );

/*
   The opposite of getHostName() :)
   Results:
   > -1 : Host found, HostID is <result>
   -1   : Database does not exist
   -2   : Wrong key
   -4   : Host not exist
*/
int getHostID(char *HostName,           /* The HostName that is searched */
              BF_KEY *Key               /* Key generated by generateKey() */
              );

/*
   Get a list of all hosts in the database
   Result is the no. of hosts in the database
   If result is < 0 an error occurred:
   -1   : Database does not exist
   -2   : Wrong key
*/
int getHosts(struct dbHosts Entrys[],   /* Define "struct dbHosts Hosts[MAX_HOSTS]" and give "Hosts" to the routine */
             BF_KEY *Key                /* Key generated by generateKey() */
             );

/*
   Add a Host to the Database
   Result is the HostID of this Host
   If result is < 0 an error occurred. see errorToStr()
*/
int addHost(char *HostName,     /* Name of new host */
            char *Comment,      /* Comment of new host */
            BF_KEY *Key         /* Key generated by generateKey() */
            );

/*
   Rename a Host
   For results see errorToStr()
*/
int modifyHost(char *OldName,           /* Old HostName */
               char *NewName,           /* New HostName */
               char *NewComment,        /* New Comment */
               BF_KEY *Key,             /* Key generated by generateKey() */
               int DeleteRoutine        /* Internal usage. Always set to 0! */
               );

/*
   Rename a Host
   For results see errorToStr()
*/
int deleteHost(char *HostName,  /* Host to delete */
               BF_KEY *Key      /* Key generated by generateKey() */
               );

/*
   Get the filename of the user database, depending of the UserName given
   Results:
   0: User not exist
   1: All okay
*/
int getUserDB(char *UserDB,     /* "char Path[256]" should be enough */
              char *UserName    /* User name. NULL for current user */
              );

/*
   Exist a user in the database and if not,
   exist an deleted user, that we can override?
   Results:
   > -1 : Host not exist, but an deleted host was found on position <result>
   -1   : Database does not exist yet
   -2   : Wrong Key
   -11  : User exist
   -12  : User not exist, no deleted host exist
*/
int existUserEntry(struct dbUsers *User, /* Only members UserName and Host are used for search */
                   BF_KEY *Key           /* Key generated by generateKey() */
                   );

/*
   Get the position of an user in the database
   Results:
   > -1 : Host was found on position <result>
   -1   : Database does not exist yet
   -2   : Wrong Key
   -12  : User not exist
*/
int findUserEntry(struct dbUsers *User, /* Only members UserName and Host are used for search.
                                           The rest will be filled if the user is found */
                  BF_KEY *Key           /* Key generated by generateKey() */
                  );

/*
   Get a list of all users in the database
   Result is the no. of users in the database
   If result is < 0 an error occurred:
   -1   : Database does not exist
   -2   : Wrong key
*/
int getUsers(struct dbUsers Entrys[],   /* Define "struct dbUsers Users[MAX_USERS]" and give "Users" to the routine */
             int HostID,                /* Only get Users with this HostID */
             BF_KEY *Key                /* Key generated by generateKey() */
             );

/*
   Add an User to the database
   For error codes see errorToStr()
*/
int addUser(char *UserName,             /* New Username */
            char *Password,             /* The Password */
            char *Comment,              /* The Comment  */
            int HostID,                 /* Assign user with this HostID */
            BF_KEY *Key                 /* Key generated by generateKey() */
            );

/*
   Modifies an User
   For error codes see errorToStr()
*/
int modifyUser(char *OldUserName,       /* Old Username */
               int OldHostID,           /* Old HostID */
               char *NewUserName,       /* New Username. If you do not want to change it, set it to NULL */
               char *NewPassword,       /* New Password. If you do not want to change it, set it to NULL */
               char *NewComment,        /* New Comment. If you do not want to change it, set it to NULL */
               int NewHostID,           /* New HostID. If you do not want to change it, set it to -1 */
               BF_KEY *Key,             /* Key generated by generateKey() */
               int DeleteRoutine        /* Internal usage. Always set to 0 */
               );

/*
   Delete an User
   For error codes see errorToStr()
*/
int deleteUser(char *UserName,          /* UserName. NULL to delete all users assigned with */
               int HostID,              /* HostID */
               BF_KEY *Key              /* Key generated by generateKey() */
               );

#endif
